from typing import List, Optional, Union

from django.utils.translation import gettext_lazy as _
from rest_framework.permissions import OperandHolder
from rest_framework.routers import DefaultRouter

from django_project_base.rest.project import ProjectSettingsViewSet, ProjectViewSet
from django_project_base.rest.project_role import ProjectRoleViewSet


def filter_rest_documentation_endpoints(endpoints: list) -> list:
    _endpoints: list = []
    for path_, path_regex, method, callback in endpoints:
        module: str = getattr(getattr(callback, "view_class", object()), "__module__", "")
        exclude: bool = "profile" in path_ and "rest_registration" in module
        if not exclude:
            _endpoints.append((path_, path_regex, method, callback))
    return _endpoints


def postprocess_rest_documentation(result, generator, request, public):
    def get_endpoint_view_function(method: str, name: str) -> Optional[object]:
        return next(
            filter(lambda e: len(e) > 3 and name in e[0] and method.lower() == e[2].lower(), generator.endpoints), None
        )

    def parse_operand_holder(holder: Union[OperandHolder, str]) -> str:
        # parse permissions like permission_classes=[IsAuthenticated | ReadOnly | ReadOnlyX]
        perm = []
        if isinstance(holder, str):
            return holder
        if isinstance(holder, OperandHolder):
            oper = ""
            p_list = []
            for op in filter(lambda o: o.endswith("_class"), dir(holder)):
                _cl = getattr(holder, op, None)
                if _cl:
                    if isinstance(_cl, OperandHolder):
                        p_list.append(parse_operand_holder(_cl))
                    else:
                        name = _cl.__name__
                        if "operator" in op:
                            oper = name
                        else:
                            p_list.append(name)
            if len(p_list) == 2 and oper:
                perm.append(f"({p_list[0]} {oper} {p_list[1]})")
            else:
                perm.append(next(iter(p_list), ""))
        return ", ".join(perm).rstrip(", ")

    def format_permission_classes(classes: List[object]) -> str:
        _classes: List[str] = []
        for cl in classes:
            if getattr(cl, "__name__", None) and cl.__name__ != "AllowAny":
                _classes.append(cl.__name__)
            elif isinstance(cl, OperandHolder):
                _classes.append(parse_operand_holder(cl))
        if not _classes:
            return ""
        return f'{_("Special permissions")}: {", ".join(_classes).rstrip(", ")}'

    for path_name, path in result.get("paths", {}).items():
        for method, data in path.items():
            try:
                view: Optional[tuple] = get_endpoint_view_function(method=method, name=path_name)
                if view:
                    _view = view[3]
                    txt: str = format_permission_classes(
                        getattr(_view, "initkwargs", {}).get("permission_classes", [])
                    ) or format_permission_classes(getattr(getattr(_view, "cls", object()), "permission_classes", []))
                    if not txt:
                        continue
                    description: str = data.get("description", "")
                    description = f"{description}\n\n{txt}"
                    data["description"] = description
            except:
                pass
    return result


class RestRouter(DefaultRouter):
    pass


# TODO: this is wrong, the location of this router. Obviously, this is a project router, so it belongs
#  to the project submodule. same goes for the rest submodule: it contains project-related stuff
django_project_base_router = RestRouter(trailing_slash=False)
django_project_base_router.register(r"project", ProjectViewSet, basename="project-base-project")
django_project_base_router.register(r"project-role", ProjectRoleViewSet, basename="project-role")
django_project_base_router.register(r"project-settings", ProjectSettingsViewSet, basename="project-settings")



import datetime
import logging

import swapper

from django.db import transaction

from django_project_base.profiling.performance_base_command import PerformanceCommand


class Command(PerformanceCommand):
    help = "Deletes users marked for deletion"

    def handle(self, *args, **options):
        for profile in swapper.load_model("django_project_base", "Profile").objects.filter(
            delete_at__isnull=False, delete_at__lt=datetime.datetime.now()
        ):
            with transaction.atomic():
                try:
                    profile.delete()
                except Exception as e:  # pragma: no cover
                    logging.getLogger(__name__).error(e)
        return "Finished deleting users"



import time

from typing import Optional

from django import db
from django.core.cache import cache

from django_project_base.celery.background_tasks.base_task import BaseTask
from django_project_base.celery.celery import app
from django_project_base.celery.settings import NOTIFICATION_QUEABLE_HARD_TIME_LIMIT, NOTIFICATION_SEND_PAUSE_SECONDS
from django_project_base.notifications.base.send_notification_service import SendNotificationService

LAST_MAIL_SENT_CK = "last-notification-was-sent-timestamp"


class SendNotificationTask(BaseTask):
    name = "background_tasks.notification_tasks.send_notification_task"

    max_retries = 0
    time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT
    soft_time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT - 3
    default_retry_delay = 0

    def run(self, notification: "DjangoProjectBaseNotification", extra_data):  # noqa: F821
        super().run()
        try:
            last_sent: Optional[float] = cache.get(LAST_MAIL_SENT_CK)
            time_from_last_sent: float = time.time() - last_sent if last_sent else 0
            if time_from_last_sent < NOTIFICATION_SEND_PAUSE_SECONDS:
                time.sleep(int(NOTIFICATION_SEND_PAUSE_SECONDS - time_from_last_sent))
            SendNotificationService(settings=self.settings).make_send(notification=notification, extra_data=extra_data)
        finally:
            cache.set(LAST_MAIL_SENT_CK, time.time(), timeout=NOTIFICATION_SEND_PAUSE_SECONDS + 1)
            db.connections.close_all()

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        super().on_failure(exc=exc, task_id=task_id, args=args, kwargs=kwargs, einfo=einfo)


send_notification_task = app.register_task(SendNotificationTask())



# Generated by Django 4.2.4 on 2023-09-06 08:48

from django.db import migrations, models
import django.db.models.deletion
import django_project_base.notifications.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0003_alter_djangoprojectbasemessage_options_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="djangoprojectbasenotification",
            options={"verbose_name": "Notification"},
        ),
        migrations.CreateModel(
            name="DeliveryReport",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        primary_key=True,
                        serialize=False,
                        verbose_name="Id",
                    ),
                ),
                ("user_id", models.CharField(db_index=True, max_length=1024)),
                ("channel", models.CharField(max_length=1024)),
                ("provider", models.CharField(db_index=True, max_length=1024)),
                ("payload", models.JSONField(null=True)),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (
                                django_project_base.notifications.models.DeliveryReport.Status["DELIVERED"],
                                "Delivered",
                            ),
                            (
                                django_project_base.notifications.models.DeliveryReport.Status["NOT_DELIVERED"],
                                "Not Delivered",
                            ),
                            (
                                django_project_base.notifications.models.DeliveryReport.Status["PENDING_DELIVERY"],
                                "Pending delivery",
                            ),
                        ],
                        db_index=True,
                        default=django_project_base.notifications.models.DeliveryReport.Status["PENDING_DELIVERY"],
                    ),
                ),
                (
                    "notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="notifications.djangoprojectbasenotification",
                    ),
                ),
            ],
        ),
    ]



import datetime

from typing import Optional

from django.conf import settings
from django.core.cache import cache

from django_project_base.base.queryset_with_cache import QuerySetWithCache
from django_project_base.notifications.base.enums import NotificationType
from django_project_base.notifications.utils import utc_now


class NotificationQuerySet(QuerySetWithCache):
    @property
    def cache_timeout(self) -> int:
        return settings.MAINTENANCE_NOTIFICATIONS_CACHE_TIMEOUT

    @property
    def base_cache_key(self) -> str:
        return settings.MAINTENANCE_NOTIFICATIONS_CACHE_KEY

    def maintenance_notifications(self):
        cached_data: Optional[list] = cache.get(self.base_cache_key)

        if cached_data is not None:
            return cached_data

        now: datetime.datetime = utc_now().timestamp()
        _data: list = super().list(
            type=NotificationType.MAINTENANCE.value,
            delayed_to__gt=now,
            delayed_to__lt=now + datetime.timedelta(hours=8).total_seconds(),
        )
        _data.sort(reverse=False, key=lambda c: c.delayed_to)
        cache.set(self.base_cache_key, _data, timeout=self.cache_timeout)
        return _data



from typing import List

from django.conf import Settings

from django_project_base.notifications.base.channels.channel import Channel, Recipient
from django_project_base.notifications.base.enums import ChannelIdentifier
from django_project_base.notifications.models import DjangoProjectBaseNotification


class SmsChannel(Channel):
    id = ChannelIdentifier.SMS.value

    name = "SMS"

    notification_price = 0.1  # TODO get from settings

    provider_setting_name = "NOTIFICATIONS_SMS_PROVIDER"

    def get_recipients(
        self, notification: DjangoProjectBaseNotification, unique_identifier="", phone_number_validator=None
    ):
        return list(
            set(
                super().get_recipients(
                    notification, unique_identifier="phone_number", phone_number_validator=phone_number_validator
                )
            )
        )

    def send(
        self, notification: DjangoProjectBaseNotification, extra_data: dict, settings: Settings, **kwargs
    ) -> int:  # noqa: F821
        return super().send(notification=notification, extra_data=extra_data, settings=settings)

    def clean_sms_recipients(self, recipients: List[Recipient]) -> List[Recipient]:
        return list(filter(lambda r: r.phone_number and len(r.phone_number), self.clean_recipients(recipients)))



from typing import Optional

import swapper

from django.db.models import Model
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _
from dynamicforms import fields
from dynamicforms.fields import AutoGeneratedField
from dynamicforms.mixins import DisplayMode
from dynamicforms.serializers import ModelSerializer
from dynamicforms.viewsets import ModelViewSet
from rest_framework.exceptions import ValidationError

from django_project_base.account.middleware import ProjectNotSelectedError
from django_project_base.auth.models import BaseRole
from django_project_base.base.models import BaseProject


class ProjectRole:
    delimiter = "§"


class ProjectRoleSerializer(ModelSerializer):
    def to_internal_value(self, data):
        internal_value = super().to_internal_value(data)
        if self.instance:
            project = self.instance.name.split(ProjectRole.delimiter)[0]
            internal_value["name"] = f'{project}{ProjectRole.delimiter}{internal_value["name"]}'
        else:
            if not internal_value.get("project"):
                raise ValidationError(dict(project=[_("Project is required")]))
            internal_value["name"] = f'{internal_value["project"]}{ProjectRole.delimiter}{internal_value["name"]}'
        internal_value.pop("project", None)
        return internal_value

    def to_representation(self, instance, row_data=None):
        project_role = super().to_representation(instance, row_data)
        project_role["name"] = project_role["name"].split(ProjectRole.delimiter)[1]
        return project_role

    project = fields.CharField(max_length=512, write_only=True, required=False)
    permissions = AutoGeneratedField(display_table=DisplayMode.SUPPRESS)

    class Meta:
        model = swapper.load_model("django_project_base", "Role")
        exclude = ()


class ProjectRoleViewSet(ModelViewSet):
    serializer_class = ProjectRoleSerializer

    def __get_project(self) -> Optional[BaseProject]:
        try:
            self.request.selected_project.get_deferred_fields()  # force immediate LazyObject evaluation
            return self.request.selected_project
        except ProjectNotSelectedError:
            pass

        if project := self.request.GET.get("project", ""):
            try:
                ProjectModel = swapper.load_model("django_project_base", "Project")
                return ProjectModel.objects.prefetch_related("owner").get(pk=project)
            except Model.DoesNotExist:
                pass

        return None

    def get_queryset(self):
        if self.request.user.is_superuser:
            return self.serializer_class.Meta.model.objects.all()
        if self.action == "list":
            project: BaseProject = self.__get_project()
            if project and project.owner_id == self.request.user.id:
                return self.serializer_class.Meta.model.objects.filter(
                    name__startswith=f"{project.pk}{ProjectRole.delimiter}"
                )
            return self.serializer_class.Meta.model.objects.none()

        try:
            role_class = swapper.load_model("django_project_base", "Role")
            role: BaseRole = get_object_or_404(
                role_class, **{self.lookup_field: self.kwargs[self.lookup_url_kwarg or self.lookup_field]}
            )
            if ProjectRole.delimiter in role.name:
                project_pk: str = role.name.split(ProjectRole.delimiter)[0]
                project: BaseProject = (
                    swapper.load_model("django_project_base", "Project").objects.filter(pk=project_pk).first()
                )
                if project and project.owner_id == self.request.user.userprofile.pk:
                    return self.serializer_class.Meta.model.objects.filter(
                        name__startswith=f"{project_pk}{ProjectRole.delimiter}"
                    )
        except:
            pass

        return self.serializer_class.Meta.model.objects.none()



import re

from abc import ABC, abstractmethod
from html import unescape
from typing import Optional, Union

import swapper

from django.conf import Settings
from django.urls import reverse
from django.utils.html import strip_tags

from django_project_base.constants import SEND_NOTIFICATION_SMS
from django_project_base.notifications.models import DeliveryReport, DjangoProjectBaseNotification


class ProviderIntegration(ABC):
    settings: Settings

    is_sms_provider = True

    def __init__(self, settings: object) -> None:
        super().__init__()
        self.settings = settings

    @abstractmethod
    def validate_send(self, response: dict):
        pass

    @abstractmethod
    def ensure_credentials(self, settings: Optional[Settings] = None):
        pass

    @abstractmethod
    def client_send(self, sender: str, recipient: "Recipient", msg: str, dlr_id: str):  # noqa: F821
        pass

    @property
    @abstractmethod
    def delivery_report_username_setting_name(self) -> str:
        pass

    @property
    @abstractmethod
    def delivery_report_password_setting_name(self) -> str:
        pass

    @abstractmethod
    def parse_delivery_report(self, dlr: DeliveryReport):
        pass

    @abstractmethod
    def ensure_dlr_user(self, project_slug: str):
        pass

    @abstractmethod
    def enqueue_dlr_request(self, pk: str):
        pass

    @abstractmethod
    def get_message(self, notification: DjangoProjectBaseNotification) -> Union[dict, str]:
        return ""

    def get_send_notification_sms_text(self, notification: DjangoProjectBaseNotification, host_url: str) -> str:
        if notification.send_notification_sms:
            template: str = (
                swapper.load_model("django_project_base", "ProjectSettings")
                .objects.get(name=SEND_NOTIFICATION_SMS, project__slug=notification.project_slug)
                .python_value
            )
            return template.replace(
                "__LINK__",
                f"{host_url.rstrip('/')}"
                f"{reverse('notification-notification-login', kwargs=dict(pk=str(notification.pk)))}",
            )
        return ""

    @abstractmethod
    def _get_sms_message(self, notification: DjangoProjectBaseNotification) -> Union[dict, str]:
        if notification.send_notification_sms:
            return notification.send_notification_sms_text

        message = f"{notification.message.subject or ''}"

        if notification.message.subject:
            message += "\n\n"

        message += (
            notification.message.body.replace("&nbsp;</p>", "\n")
            .replace("</p>", "\n")
            .replace("\n&nbsp;", "\n")
            .replace("&nbsp;", "\n")
        )
        text_only = re.sub("[ \t]+", " ", strip_tags(message))
        # Strip single spaces in the beginning of each line
        message = text_only.replace("\n ", "\n")
        message = message.replace("\n", "\r\n")
        return unescape(message)



import datetime

from typing import Any, Dict

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from django.utils.crypto import get_random_string
from django.utils.translation import gettext as __
from natural.date import compress
from rest_framework.request import Request
from rest_registration.exceptions import UserNotFound
from rest_registration.signers.reset_password import ResetPasswordSigner
from rest_registration.utils.users import get_user_verification_id

from django_project_base.account.constants import RESET_USER_PASSWORD_VERIFICATION_CODE
from django_project_base.notifications.email_notification import EMailNotification
from django_project_base.notifications.models import DjangoProjectBaseMessage


def send_reset_password_verification_email(request: Request, user, send=False) -> Dict:
    if not send:
        return {}
    signer = ResetPasswordSigner(
        {
            "user_id": get_user_verification_id(user),
        },
        request=request,
    )

    code_ck = RESET_USER_PASSWORD_VERIFICATION_CODE + str(user.pk)
    code = get_random_string(length=6)
    cache.set(code_ck, code, timeout=settings.CONFIRMATION_CODE_TIMEOUT)

    try:
        project = request.selected_project.slug
    except:
        project = None

    EMailNotification(
        message=DjangoProjectBaseMessage(
            subject=f"{__('Password recovery for')} {request.META['HTTP_HOST']}",
            body=f"{__('You or someone acting as you requested a password reset for your account at')} "
            f"{request.META['HTTP_HOST']}. "
            f"\n\n{__('Your verification code is')}: "
            f"{code} \n\n {__('Code is valid for')} {compress(settings.CONFIRMATION_CODE_TIMEOUT)}.\n\n"
            f"{__('If this was not you or it was unintentional, you may safely ignore this message.')}",
            footer="",
            content_type=DjangoProjectBaseMessage.PLAIN_TEXT,
        ),
        raw_recipents=[user.pk],
        project=project,
        delay=int(datetime.datetime.now().timestamp()),
        recipients=[user.pk],
        user=request.user.pk,
    ).send()

    return signer.get_signed_data()


def find_user_by_send_reset_password_link_data(data: Dict[str, Any], **kwargs: Any):
    query = kwargs["serializer"].validated_data if kwargs.get("serializer", None) else data
    user = get_user_model().objects.filter(**query).first()
    if user:
        return user
    raise UserNotFound()



# Generated by Django 4.2.4 on 2023-08-28 08:57

from django.db import migrations, models, connection


def forwards_func(apps, schema_editor):
    from django_project_base.notifications.models import DjangoProjectBaseNotification

    if list(
        filter(
            lambda g: getattr(g, "column", "") in ("project", "project_id"),
            DjangoProjectBaseNotification._meta.get_fields(),
        )
    ):
        try:
            with connection.cursor() as cursor:
                cursor.execute("ALTER TABLE notifications_djangoprojectbasenotification drop column project_id;")
        except Exception:
            pass


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
        migrations.AddField(
            model_name="djangoprojectbasenotification",
            name="project_slug",
            field=models.CharField(blank=True, max_length=1024, null=True),
        ),
    ]



import threading

from typing import Optional

from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest

_threadmap = {}


def has_current_request() -> bool:
    return threading.get_ident() in _threadmap


def get_current_request() -> WSGIRequest:
    return _threadmap[threading.get_ident()]


def get_parameter(request, value_name: str, url_part: str) -> Optional[object]:
    value_from_header: Optional[int] = request.headers.get(f"Current-{value_name.lower().title()}")
    if value_from_header is not None and value_from_header not in ("", "null"):
        return value_from_header

    path_parts = request.path_info.split("/")
    if isinstance(url_part, (list, tuple)) and isinstance(url_part[0], int) and isinstance(url_part[1], (list, tuple)):
        parm = path_parts[url_part[0]] if len(path_parts) > url_part[0] else None
        return parm if parm not in url_part[1] else None

    try:
        project_info = next(iter(filter(lambda f: f and url_part in f, path_parts)))
    except StopIteration:
        project_info = None
    if project_info:
        url_part_len = len(url_part)
        return project_info[url_part_len:]

    return None


class UrlVarsMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        for value, config in settings.DJANGO_PROJECT_BASE_BASE_REQUEST_URL_VARIABLES.items():
            if param := get_parameter(request, value, config["url_part"]):
                setattr(request, config["value_name"], param)

        _threadmap[threading.get_ident()] = request

        response = self.get_response(request)

        del _threadmap[threading.get_ident()]

        return response



from typing import List


class PhoneNumberParser:
    @staticmethod
    def is_allowed(phone_number: str, allowed_validator=None) -> bool:
        if allowed_validator:
            return allowed_validator(phone_number)
        return phone_number and len(phone_number) >= 8

    @staticmethod
    def valid_phone_numbers(candidates: List[str], allowed_validator=None) -> List[str]:
        valid: List[str] = []
        for number in candidates:
            if not PhoneNumberParser.is_allowed(number, allowed_validator):
                continue
            if number.startswith("+"):
                valid.append(number.lstrip("+"))
                continue
            if number.startswith("00"):
                valid.append(number.lstrip("00"))
                continue
            valid.append(number)
        return valid

    @staticmethod
    # TODO: SLOVENIA????????
    def ensure_country_code_slovenia(candidates: List[str]) -> List[str]:
        valid: List[str] = []
        for number in candidates:
            if number.startswith("+"):
                valid.append(number)
                continue
            if number.startswith("00"):
                nm = number.lstrip("00")
                valid.append(f"+{nm}")
                continue
            if number.startswith("0"):
                nm = number.lstrip("0")
                valid.append(f"+386{nm}")
                continue
            if number.startswith("386"):
                valid.append(f"+{number}")
                continue
            valid.append(f"+386{number}")
        return valid



from rest_framework import status
from rest_framework.exceptions import APIException


class LicenseActionNotImplementedException(APIException):
    status_code = status.HTTP_501_NOT_IMPLEMENTED


class InviteActionNotImplementedException(APIException):
    status_code = status.HTTP_501_NOT_IMPLEMENTED



import swapper

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.core.cache import cache
from django.db.models.signals import post_delete, post_save

from django_project_base.settings import USER_CACHE_KEY


def user_cache_invalidate(instance):
    if isinstance(instance, get_user_model()):
        instance_user_id = instance.id
    else:
        instance_user_id = instance.user_id

    cache.delete(USER_CACHE_KEY.format(id=instance_user_id))


def invalidate_cache(sender, instance, **kwargs):
    user_cache_invalidate(instance)


class UsersBackend(ModelBackend):
    def get_user(self, user_id):
        try:
            user = swapper.load_model("django_project_base", "Profile").objects.get(pk=user_id)
        except Exception:
            return None
        return user if self.user_can_authenticate(user) else None


class UsersCachingBackend(UsersBackend):
    def __init__(self) -> None:
        super().__init__()
        post_save.connect(invalidate_cache, sender=get_user_model())
        post_delete.connect(invalidate_cache, sender=get_user_model())
        # even though the password is changed in Django user model, it is still UserProfile model that is saved
        #  and thus the signal for Django user model isn't firing
        post_save.connect(invalidate_cache, sender=swapper.load_model("django_project_base", "Profile"))
        post_delete.connect(invalidate_cache, sender=swapper.load_model("django_project_base", "Profile"))

    def get_user(self, user_id):
        user = cache.get(USER_CACHE_KEY.format(id=user_id or 0))
        if not user:
            user = super().get_user(user_id)
            if user_id and user:
                cache.set(USER_CACHE_KEY.format(id=user_id or 0), user)
        return user



#!/bin/python3
import fileinput
import os
import sys

import setuptools

from django_project_base import __version__

publish_local = True


def write_ver_to_init(file_name: str, version: str, search: str, replacement: str):
    replacement = replacement % version
    filename = file_name
    for line in fileinput.input([filename], inplace=True):
        if line.strip().startswith(search):
            line = replacement
        sys.stdout.write(line)


def get_version(version_arg):
    from versio.version import Version
    from versio.version_scheme import Simple3VersionScheme

    try:
        if version_arg == "publish":
            print("Missing version argument.")
            sys.exit(1)
        Version(version_arg, scheme=Simple3VersionScheme)
    except Exception:
        print("Invalid version format. Should be x.y.z (all numbers)")
        sys.exit(1)
    return version_arg


with open("README.md", "r") as fh:
    long_description = fh.read()
with open("requirements.txt", "r") as fh:
    requirements = fh.readlines()

for lnum, line in enumerate(requirements):
    if line[:3] == "git":
        requirements.pop(lnum)

version = __version__

if sys.argv[1] == "publish":
    version_str = get_version(sys.argv[-1])

    if os.system("python -m wheel version"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    if os.system("python -m twine --version"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    if os.system("tox -e check") or os.system("npm run test"):
        sys.exit()

    write_ver_to_init("django_project_base/__init__.py", version_str, "__version__", "__version__ = '%s'\n")
    write_ver_to_init("package.json", version_str, '"version": ', '  "version": "%s",\n')

    os.system("npm run build")
    os.system("npm publish --access=public")
    os.system("rm -rf build && rm -rf dist && rm -rf django_project_base.egg-info")

    os.system("python setup.py sdist bdist_wheel")
    # if you don't like to enter username / pass for pypi every time, run this command:
    #  keyring set https://upload.pypi.org/legacy/ username  (it will ask for password)
    os.system("twine upload dist/*")

    os.system("rm -rf build && rm -rf dist && rm -rf django_project_base.egg-info")
    os.system("git checkout django_project_base/__init__.py")
    os.system("git checkout package.json")
    os.system("git tag -a %s -m 'version %s'" % (version_str, version_str))
    os.system("git push --tags")
    sys.exit()

setuptools.setup(
    name="django-project-base",
    version=version,
    author="Jure Erznožnik",
    author_email="jure@velis.si",
    description="Everything revolves around it: users, roles, permissions, tags, etc.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/velis74/django-project-base",
    packages=setuptools.find_packages(include=("django_project_base",)),
    include_package_data=True,
    install_requires=requirements,
    python_requires=">=3.8",
    license="BSD-3-Clause",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
    ],
)



# Generated by Django 4.2.4 on 2023-09-01 06:13

from django.db import migrations


def forwards_func(apps, schema_editor):
    apps.get_model("licensing", "LicenseAccessUse").objects.all().update(comment=None)


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        (
            "licensing",
            "0002_remove_licenseaccessuse_comment_and_more_squashed_0004_alter_licenseaccessuse_amount",
        ),
    ]

    operations = [migrations.RunPython(forwards_func, reverse_func)]



__title__ = "django project base"
# __version__ = "{version-string}"  #TODO: Switch back to version string when merging to main
__version__ = '0.76.21'
__author__ = "Jure Erznožnik"
__email__ = "jure.erznoznik@gmail.com"
__license__ = "BSD 3-Clause"
__copyright__ = "Copyright 2020-present Jure Erznožnik"
default_app_config = "django_project_base.apps.DjangoProjectBaseConfig"

VERSION = __version__



import socket
import uuid
from typing import List, Type

import swapper
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.test import TransactionTestCase
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient

from django_project_base.constants import USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER
from django_project_base.licensing.models import LicenseAccessUse
from django_project_base.notifications.base.channels.channel import Channel
from django_project_base.notifications.base.channels.mail_channel import MailChannel
from django_project_base.notifications.base.notification import Notification
from django_project_base.notifications.models import DjangoProjectBaseMessage, DjangoProjectBaseNotification


class TestNotificationViaEmail(Notification):
    @property
    def via_channels(self) -> List[Type[Channel]]:
        return [MailChannel]


class NotificationsTransactionTestCase(TransactionTestCase):
    api_client: APIClient

    @property
    def test_user(self):
        return get_user_model().objects.get(username="testuser")

    def setUp(self) -> None:
        super().setUp()
        self.__createUser()
        swapper.load_model("django_project_base", "Project").objects.create(
            name="test", slug="test", owner=self.test_user.userprofile
        )
        self.api_client = APIClient()
        LicenseAccessUse.objects.create(
            type=LicenseAccessUse.UseType.USE,
            user_id=str(self.test_user.pk),
            content_type_object_id=str(uuid.uuid4()),
            content_type=ContentType.objects.get_for_model(DjangoProjectBaseNotification._meta.model),
            amount=-100,
            comment=dict(comment="Test Credit", count=0, item_price=0, sender=""),
        )
        for p in swapper.load_model("django_project_base", "Project").objects.all():
            swapper.load_model("django_project_base", "ProjectSettings").objects.get_or_create(
                name=USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER,
                project=p,
                defaults=dict(
                    value=False,
                    value_type="char",
                    description="Tests value",
                ),
            )

    def _login_to_api_client_with_test_user(self):
        user_token, token_created = Token.objects.get_or_create(user=self.test_user)
        self.api_client.credentials(
            HTTP_AUTHORIZATION="Token " + user_token.key, HTTP_HOST=socket.gethostname().lower()
        )

    def __createUser(self):
        get_user_model().objects.get_or_create(
            **dict(
                username="testuser",
                email="testuser@testuser.testuser",
                first_name="testuser",
                last_name="testuser",
            )
        )

    def create_notification_email_object(self) -> TestNotificationViaEmail:
        return TestNotificationViaEmail(
            message=DjangoProjectBaseMessage.objects.create(subject="Test mail", body="content"),
            raw_recipents=[self.test_user.pk],
            project=swapper.load_model("django_project_base", "Project").objects.first().slug,
            recipients=[self.test_user.pk],
            persist=True,
            user=self.test_user.pk,
        )



import json

from typing import Optional

import social_core.backends.facebook
import social_core.backends.google
import social_core.backends.microsoft

from django.db import connection
from social_django.models import UserSocialAuth

from django_project_base.profiling.performance_base_command import PerformanceCommand


# This is not tested because there is no socialaccount model anymore
class Command(PerformanceCommand):  # pragma: no cover
    help = "Migrate social login data from allauth to social_core"

    provider_mapping: dict = {
        "google": social_core.backends.google.GoogleOAuth2.name,
        "facebook": social_core.backends.facebook.FacebookOAuth2.name,
        "azure": social_core.backends.microsoft.MicrosoftOAuth2.name,
    }

    def handle(self, *args, **options):
        with connection.cursor() as cursor:
            cursor.execute("select user_id, provider, extra_data from socialaccount_socialaccount;")
            rows: list = cursor.fetchall()
            for row in rows:
                user_id: int = row[0]
                provider: str = row[1]
                extra_data: dict = json.loads(row[2]) if row[2] else {}
                user_email: Optional[str] = extra_data.get("email") or extra_data.get("mail")
                if user_email:
                    new_provider: str = self.provider_mapping.get(provider)
                    assert new_provider, "New social auth provider for %s is not defined" % provider
                    payload = {
                        "user_id": user_id,
                        "provider": new_provider,
                    }
                    social_auth_records_exists: bool = UserSocialAuth.objects.filter(**payload).exists()
                    if not social_auth_records_exists:
                        payload["uid"] = user_email
                        payload["extra_data"] = None
                        UserSocialAuth.objects.create(**payload)
        return "ok"



import datetime
import logging
import uuid

from abc import ABC, abstractmethod
from typing import List, Optional, Tuple, Union

from django.conf import Settings, settings
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string
from django.utils.translation import gettext

from django_project_base.notifications.base.channels.integrations.provider_integration import ProviderIntegration
from django_project_base.notifications.base.phone_number_parser import PhoneNumberParser
from django_project_base.notifications.models import (
    DeliveryReport,
    DjangoProjectBaseMessage,
    DjangoProjectBaseNotification,
)
from django_project_base.utils import get_pk_name


class Recipient:
    identifier: str
    phone_number: str
    email: str
    unique_attribute: str

    def __init__(
        self,
        identifier: str,
        phone_number: str,
        email: str,
        unique_attribute: str = "identifier",
        phone_number_validator=None,
    ) -> None:
        super().__init__()
        self.identifier = identifier
        self.phone_number = (
            next(
                iter(
                    PhoneNumberParser.valid_phone_numbers([phone_number], phone_number_validator)
                    if phone_number and len(phone_number)
                    else ""
                ),
                None,
            )
            or ""
        )
        self.email = email
        self.unique_attribute = unique_attribute

    def __eq__(self, __o: "Recipient") -> bool:
        return str(getattr(self, self.unique_attribute)) == str(getattr(__o, self.unique_attribute))

    def __hash__(self) -> int:
        return getattr(self, self.unique_attribute).__hash__()


class Channel(ABC):
    id = None

    name = ""

    notification_price = 0

    provider_setting_name = ""

    _provider: ProviderIntegration

    def _get_provider(self):
        return self._provider

    def _set_provider(self, val: ProviderIntegration):
        self._provider = val

    provider = property(_get_provider, _set_provider)

    def sender(self, notification: DjangoProjectBaseNotification) -> str:
        _sender = getattr(notification, "sender", {}).get(self.name)
        if not getattr(settings, "TESTING", False):
            assert _sender, "Notification sender is required"
        return _sender

    def _find_provider(
        self, settings: Optional[Settings], setting_name: str, exclude: Optional[List[str]] = None
    ) -> Optional[ProviderIntegration]:
        if exclude is None:
            exclude = []

        def get_first_provider(val: Union[str, List]):
            if val and isinstance(val, (list, tuple)):
                prov = next(filter(lambda i: i not in exclude, val), None)

                return import_string(prov)() if prov else None

            return import_string(val)() if val not in exclude and val else None

        if settings and getattr(settings, setting_name, None):
            return get_first_provider(getattr(settings, setting_name))
        return get_first_provider(getattr(settings or object(), setting_name, ""))

    def clean_recipients(self, recipients: List[Recipient]) -> List[Recipient]:
        return list(set(recipients))

    def create_delivery_report(
        self,
        notification: DjangoProjectBaseNotification,
        recipient: Recipient,
        pk: str,
        channel: Optional[str] = None,
        provider: Optional[str] = None,
        auxiliary_notification: Optional[uuid.UUID] = None,
    ) -> DeliveryReport:
        return next(
            iter(
                DeliveryReport.objects.get_or_create(
                    notification=notification,
                    user_id=recipient.identifier,
                    channel=f"{self.__module__}.{self.__class__.__name__}" if not channel else channel,
                    provider=f"{self.provider.__module__}.{self.provider.__class__.__name__}"
                    if not provider
                    else provider,
                    pk=pk,
                    auxiliary_notification=auxiliary_notification,
                )
            ),
            None,
        )

    @abstractmethod
    def get_recipients(
        self, notification: DjangoProjectBaseNotification, unique_identifier="email", phone_number_validator=None
    ) -> List[Recipient]:
        rec_obj = notification.email_list
        if not rec_obj:
            rec_obj = notification.recipients_list
        if not rec_obj:
            att = ("email", "phone_number", get_pk_name(get_user_model()))
            rec_obj = [
                {k: v for k, v in profile.__dict__.items() if not k.startswith("_") and k in att}
                for profile in [
                    get_user_model().objects.get(pk=u).userprofile for u in notification.recipients.split(",")
                ]
            ]
        return [
            Recipient(
                identifier=u.get("id", "") or "",
                email=u.get("email", "") or "",
                phone_number=u.get("phone_number", "") or "",
                unique_attribute=unique_identifier,
                phone_number_validator=phone_number_validator,
            )
            for u in rec_obj
        ]

    def _make_send(self, notification_obj, rec_obj, message_str, dlr_pk) -> Tuple[Optional[DeliveryReport], bool]:
        do_send = True
        sent = False
        logger = logging.getLogger("django")
        try:
            dlr_exists = list(
                DeliveryReport.objects.filter(
                    notification=notification_obj,
                    user_id=rec_obj.identifier,
                    channel=f"{self.__module__}.{self.__class__.__name__}",
                    provider=f"{self.provider.__module__}.{self.provider.__class__.__name__}",
                )
            )
            if len(dlr_exists) > 1:
                raise Exception(f"{gettext('To many DLR exist.')} {notification_obj} {rec_obj}")
            if dlr_notification := next(iter(dlr_exists), None):
                dlr_pk = str(dlr_notification.pk)
                if dlr_notification.status == DeliveryReport.Status.DELIVERED:
                    do_send = False

            if getattr(settings, "TESTING", False):
                sent = True
            elif do_send:
                self.provider.client_send(self.sender(notification_obj), rec_obj, message_str, dlr_pk)
                sent = True
        except Exception as te:
            logger.exception(te)
            sent = False
        dlr_obj = None
        try:
            if sent and do_send:
                dlr_obj = self.create_delivery_report(notification_obj, rec_obj, dlr_pk)
            return dlr_obj, (sent and do_send)
        except Exception as de:
            logger.exception(de)
            return dlr_obj, (sent and do_send)

    def send(self, notification: DjangoProjectBaseNotification, extra_data, settings: Settings, **kwargs) -> int:
        logger = logging.getLogger("django")
        try:
            message = self.provider.get_message(notification)

            recipients = self.get_recipients(
                notification, phone_number_validator=getattr(settings, "IS_PHONE_NUMBER_ALLOWED_FUNCTION", None)
            )

            if not recipients:
                raise ValueError("No valid recipients")

            exclude_providers: List[str] = []
            sent_no = 0

            from django_project_base.notifications.base.channels.mail_channel import MailChannel

            mail_fallback = (
                MailChannel.name not in (notification.required_channels or "").split(",")
                and notification.email_fallback
            )

            from django_project_base.notifications.email_notification import EMailNotification

            for recipient in recipients:  # noqa: E203
                dlr__uuid = str(uuid.uuid4())
                was_sent = False
                try:
                    if (
                        self.provider.is_sms_provider
                        and not recipient.phone_number
                        and mail_fallback
                        and not notification.send_notification_sms
                    ):
                        try:
                            a_notification = EMailNotification(
                                message=DjangoProjectBaseMessage(
                                    subject=notification.message.subject,
                                    body=notification.message.body,
                                    footer=notification.message.footer,
                                    content_type=notification.message.content_type,
                                ),
                                raw_recipents=[
                                    recipient.identifier,
                                ],
                                project=notification.project_slug if notification.project_slug else None,
                                recipients=[
                                    recipient.identifier,
                                ],
                                a_sender=notification.sender,
                                a_extra_data=extra_data,
                                a_recipients_list=notification.recipients_list,
                                a_settings=settings,
                                delay=int(datetime.datetime.now().timestamp()),
                                user=extra_data["user"],
                            ).send()
                            self.create_delivery_report(
                                notification, recipient, dlr__uuid, auxiliary_notification=a_notification.pk
                            )
                            continue
                        except Exception as e:
                            logger.exception(e)
                            continue

                    while True:
                        _make_send = self._make_send(
                            notification_obj=notification, message_str=message, rec_obj=recipient, dlr_pk=dlr__uuid
                        )
                        if _make_send[1]:
                            dlr__uuid = str(_make_send[0].pk)
                            was_sent = True
                            break
                        else:
                            exclude_providers.append(f"{self.provider.__module__}.{self.provider.__class__.__name__}")
                            if next_provider := self._find_provider(
                                settings=settings,
                                setting_name=self.provider_setting_name,
                                exclude=exclude_providers,
                            ):
                                self.provider = next_provider
                            else:
                                break
                    if was_sent:
                        self.provider.enqueue_dlr_request(pk=dlr__uuid)
                        sent_no += 1
                except Exception as ge:
                    logger.exception(ge)

            if self.provider.is_sms_provider:
                from django_project_base.notifications.base.channels.integrations.t2 import SMSCounter

                sent_no = sent_no * SMSCounter.count(message)["messages"]
            return sent_no
        except Exception as e:
            logger.exception(e)
            raise e



from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from django_project_base.base.exceptions import LicenseActionNotImplementedException
from django_project_base.licensing.logic import LogAccessService


class LicenseViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]

    def list(self, request):
        return Response(LogAccessService().report(user=request.user))

    def create(self, request):
        raise LicenseActionNotImplementedException()

    def retrieve(self, request, pk=None):
        raise LicenseActionNotImplementedException()

    def update(self, request, pk=None):
        raise LicenseActionNotImplementedException()

    def partial_update(self, request, pk=None):
        raise LicenseActionNotImplementedException()

    def destroy(self, request, pk=None):
        raise LicenseActionNotImplementedException()



from typing import List

import swapper

from django.core.cache import cache
from django.db import transaction
from django.utils.translation import gettext_lazy as _
from dynamicforms import fields
from dynamicforms.action import Actions, TableAction, TablePosition
from dynamicforms.mixins import DisplayMode
from dynamicforms.template_render.responsive_table_layout import ResponsiveTableLayout, ResponsiveTableLayouts
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.exceptions import APIException, ValidationError
from rest_framework.fields import IntegerField, ListField
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import Serializer

from django_project_base.account.constants import MERGE_USERS_QS_CK
from django_project_base.account.rest.profile import ProfileSerializer, ProfileViewSet
from django_project_base.base.permissions import IsProjectOwner, IsSuperUser
from django_project_base.utils import get_pk_name


class ProfileMergeSerializer(ProfileSerializer):
    template_context = dict(url_reverse="profile-merge-base-project")

    form_titles = {
        "table": "Merge users",
        "new": "",
        "edit": "",
    }

    actions = Actions(add_form_buttons=False)

    phone_number = fields.CharField(display=DisplayMode.HIDDEN)
    language = fields.CharField(display=DisplayMode.HIDDEN)
    email = fields.CharField(display=DisplayMode.HIDDEN)

    def __init__(self, *args, is_filter: bool = False, **kwds):
        super().__init__(*args, is_filter=is_filter, **kwds)
        self.actions.actions = (
            TableAction(
                TablePosition.ROW_END,
                label=_("Remove"),
                title=_("Remove"),
                name="delete-user",
                icon="person-remove-outline",
            ),
            TableAction(
                TablePosition.HEADER,
                label=_("Merge"),
                title=_("Merge"),
                name="merge-users",
                icon="git-merge-outline",
            ),
            TableAction(
                TablePosition.HEADER,
                label=_("Clear all"),
                title=_("Clear all"),
                name="clear-merge-users",
                icon="remove-circle-outline",
            ),
        )

    @property
    def filter_data(self):
        return None

    class Meta(ProfileSerializer.Meta):
        responsive_columns = ResponsiveTableLayouts(
            auto_generate_single_row_layout=True,
            layouts=[
                ResponsiveTableLayout(auto_add_non_listed_columns=True),
                ResponsiveTableLayout("full_name", auto_add_non_listed_columns=False),
            ],
        )


class MergeUsersRequest(Serializer):
    users = ListField(child=IntegerField(min_value=1), required=True, allow_empty=False, min_length=2)

    def validate(self, attrs):
        MergeUserGroup = swapper.load_model("django_project_base", "MergeUserGroup")
        for user in attrs["users"]:
            if str(user) in ",".join(MergeUserGroup.objects.values_list("users", flat=True)).split(","):
                raise ValidationError(dict(users=f"Pk {user} is present in another group of users to be merged"))
        return super().validate(attrs)


class ProfileMergeViewSet(ProfileViewSet):
    serializer_class = ProfileMergeSerializer
    permission_classes = [IsAuthenticated, IsAdminUser | IsSuperUser | IsProjectOwner]

    schema = None

    def get_queryset(self) -> List:
        ck_val = cache.get(MERGE_USERS_QS_CK % self.request.user.pk, [])
        if not ck_val:
            return super().get_queryset().filter(pk=-1)
        return super().get_queryset().filter(pk__in=ck_val)

    def get_serializer_class(self):
        return ProfileMergeSerializer

    def get_permissions(self):
        if self.action == "clear":
            return [IsAuthenticated(), IsAdminUser()]
        else:
            return super().get_permissions()

    @transaction.atomic
    def create(self, request: Request, *args, **kwargs) -> Response:
        MergeUserGroup = swapper.load_model("django_project_base", "MergeUserGroup")
        ck = MERGE_USERS_QS_CK % self.request.user.pk
        ser = MergeUsersRequest(data=dict(users=cache.get(ck, [])))
        ser.is_valid(raise_exception=True)
        group, created = MergeUserGroup.objects.get_or_create(
            users=",".join(map(str, ser.validated_data["users"])),
            created_by=self.request.user.pk,
            project=request.selected_project,
        )
        cache.set(ck, [])
        return Response({get_pk_name(MergeUserGroup): group.pk})

    def get_current_profile(self, request: Request, **kwargs) -> Response:
        raise APIException(code=status.HTTP_501_NOT_IMPLEMENTED)

    def mark_current_profile_delete(self, request: Request, **kwargs) -> Response:
        raise APIException(code=status.HTTP_501_NOT_IMPLEMENTED)

    def destroy(self, request, *args, **kwargs):
        pk = self.get_object().pk
        ck = MERGE_USERS_QS_CK % self.request.user.pk
        ck_val = cache.get(ck, [])
        ck_val = [i for i in ck_val if i != pk]
        cache.set(ck, list(set(ck_val)), timeout=None)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def partial_update(self, request, *args, **kwargs):
        raise APIException(code=status.HTTP_501_NOT_IMPLEMENTED)

    @action(methods=["DELETE"], detail=False, url_path="clear", url_name="clear")
    def clear(self, request: Request, **kwargs) -> Response:
        MergeUserGroup = swapper.load_model("django_project_base", "MergeUserGroup")

        cache.set(MERGE_USERS_QS_CK % self.request.user.pk, [])
        MergeUserGroup.objects.filter(created_by=self.request.user.pk).delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



import functools

from celery import shared_task

from django_project_base.profiling.performance_function_decorator import function_profiler


def shared_task_profiler(*args, **kwargs):
    profiler_name = kwargs.pop("profiler_name", None)

    def decorator(func):
        profiled_func = function_profiler(name=profiler_name)(func)

        # Then, define a new function that applies shared_task to the wrapped function
        @shared_task(*args, **kwargs)
        @functools.wraps(profiled_func)
        def task_wrapper(*_args, **_kwargs):
            return profiled_func(*_args, **_kwargs)

        return task_wrapper

    return decorator



import logging

from typing import Optional

from django.conf import Settings
from django.db import connections
from django.db.utils import load_backend

from django_project_base.celery.settings import NOTIFICATION_QUEABLE_HARD_TIME_LIMIT
from django_project_base.constants import NOTIFICATION_QUEUE_NAME
from django_project_base.profiling.performance_celery_task_class import PerformanceCeleryTask

LAST_MAIL_SENT_CK = "last-notification-was-sent-timestamp"


class BaseTask(PerformanceCeleryTask):
    name = ""

    max_retries = 0
    time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT
    soft_time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT - 3
    default_retry_delay = 0

    settings: Optional[Settings] = None

    def before_start(self, task_id, args, kwargs):
        if (path := self._app.conf.get("django-settings-module")) and len(path):
            self.settings = Settings(path)
            db_settings: dict = self.settings.DATABASES["default"]
            db_settings.setdefault("TIME_ZONE", None)
            db_settings.setdefault("CONN_HEALTH_CHECKS", None)
            db_settings.setdefault("CONN_MAX_AGE", 0)
            db_settings.setdefault("OPTIONS", {})
            db_settings.setdefault("AUTOCOMMIT", True)
            backend = load_backend(db_settings["ENGINE"])
            dw = backend.DatabaseWrapper(db_settings)
            dw.connect()
            connections.databases[NOTIFICATION_QUEUE_NAME] = dw.settings_dict
            connections.databases["default"] = dw.settings_dict

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        logging.getLogger(__name__).error(
            f"Exception: {exc} \n\nTask id: {task_id}\n\nArgs: {args}\n\nKwargs: {kwargs}\n\nEInfo: {einfo}"
        )

    def run(self, *args, **kwargs):
        return None



from collections.abc import Iterable

import pytz
import swapper

from django.conf import settings
from django.db import models
from django.db.models import Case, CharField, Prefetch, QuerySet, Value, When
from django.db.models.functions import Coalesce, Concat
from django.utils.dateparse import (
    datetime_re,
    iso8601_duration_re,
    parse_datetime,
    parse_duration,
    parse_time,
    standard_duration_re,
    time_re,
)
from django.utils.timezone import datetime, now, timedelta
from dynamicforms import fields
from rest_framework.request import Request

from django_project_base.account.middleware import ProjectNotSelectedError


def get_project_members(request: Request, project=None) -> QuerySet:
    project = project or request.selected_project
    try:
        project_members = swapper.load_model("django_project_base", "ProjectMember").objects.filter(project_id=project)
    except ProjectNotSelectedError:
        project = None
        project_members = swapper.load_model("django_project_base", "ProjectMember").objects.all()
    qs = (
        swapper.load_model("django_project_base", "Profile")
        .objects.prefetch_related(Prefetch("projects", queryset=project_members), "groups", "user_permissions")
        .annotate(
            un=Concat(
                Coalesce(
                    Case(When(first_name="", then="username"), default="first_name", output_field=CharField()),
                    Value(""),
                ),
                Value(" "),
                Coalesce(
                    Case(When(last_name="", then="username"), default="last_name", output_field=CharField()),
                    "username",
                ),
            ),
            un_sort=Concat(
                Coalesce(
                    Case(When(last_name="", then="username"), default="last_name", output_field=CharField()),
                    "username",
                ),
                Value(" "),
                Coalesce(
                    Case(When(first_name="", then="username"), default="first_name", output_field=CharField()),
                    Value(""),
                ),
            ),
        )
    )

    qs = qs.exclude(delete_at__isnull=False, delete_at__lt=now())

    if project is not None:
        # if current project was parsed from request, filter profiles to current project only
        qs = qs.filter(projects__project=project)
    else:
        qs = qs.none()

    if request.query_params.get("remove-merge-users", "false") in fields.BooleanField.TRUE_VALUES:
        MergeUserGroup = swapper.load_model("django_project_base", "MergeUserGroup")
        exclude_qs = set()
        for mgu in MergeUserGroup.objects.filter(created_by=request.user.pk).values_list("users", flat=True).all():
            if mgu and mgu.split(","):
                exclude_qs.update(map(int, mgu.split(",")))
        if exclude_qs:
            qs = qs.exclude(pk__in=exclude_qs)

    qs = qs.order_by("un", "id")
    return qs.distinct()


def __get_time_resolution(value: str) -> dict:
    try:
        value = value.replace("Z", "")
        resolution = (
            getattr(time_re.match(value), "lastgroup", "")
            or getattr(datetime_re.match(value), "lastgroup", "")
            or getattr(standard_duration_re.match(value), "lastgroup", "")
            or getattr(iso8601_duration_re.match(value), "lastgroup", "")
        )
        if resolution in ("second", "microsecond", "seconds", "microseconds"):
            return dict(seconds=1)
        if resolution in ("hour", "hours"):
            return dict(hours=1)
        if resolution in ("minute", "minutes"):
            return dict(minutes=1)
    except:
        pass
    return {}


def filter_project_members_fields(queryset: QuerySet, field: str, value) -> QuerySet:
    if value is None or value == "":
        return queryset

    if field == "full_name":
        return queryset.filter(un__icontains=value)
    if field == "state":
        return queryset.filter(projects__state=value)
    if field == "user_groups":
        if isinstance(value, Iterable):
            return queryset.filter(members__user_group__in=value)
        return queryset.filter(members__user_group=value)

    model_meta = queryset.model._meta

    if field not in (fld.name for fld in model_meta.get_fields()):
        return queryset

    try:
        # TODO: this would probably be better moved into the fields themselves
        if isinstance(model_meta.get_field(field), (models.CharField, models.TextField)):
            return queryset.filter(**{field + "__icontains": value})
        if isinstance(model_meta.get_field(field), (models.DateTimeField,)):
            date_time: datetime = parse_datetime(value.replace("Z", ""))
            date_time.replace(microsecond=0)
            date_time = pytz.timezone(settings.TIME_ZONE).localize(date_time).astimezone(pytz.utc)
            return queryset.filter(
                **{
                    field + "__gte": date_time,
                    field + "__lt": date_time + timedelta(**__get_time_resolution(value)),
                }
            )
        if isinstance(model_meta.get_field(field), (models.TimeField,)):
            start = datetime.combine(now().date(), parse_time(value).replace(microsecond=0))
            end = (start + timedelta(**__get_time_resolution(value))).time()
            return queryset.filter(**{field + "__gte": start, field + "__lt": end})
        if isinstance(model_meta.get_field(field), (models.DurationField,)):
            duration = parse_duration(value)
            duration = duration - timedelta(microseconds=duration.microseconds)
            return queryset.filter(
                **{
                    field + "__gte": duration,
                    field + "__lt": duration + timedelta(**__get_time_resolution(value)),
                }
            )
        if isinstance(model_meta.get_field(field), (models.DateField,)):
            date_time = None
            for date_time_fmt in [settings.DATE_FORMAT, "%Y-%m-%d"]:
                try:
                    date_time = datetime.strptime(value, date_time_fmt)
                    break
                except:
                    pass
            date_time = pytz.timezone(settings.TIME_ZONE).localize(date_time).astimezone(pytz.utc)
            return queryset.filter(**{field + "__gte": date_time, field + "__lt": date_time + timedelta(days=1)})
        else:
            if isinstance(model_meta.get_field(field), models.BooleanField):
                value = value == "true"
            return queryset.filter(**{field: value})
    except:
        return queryset



import datetime
import json
import uuid

from typing import List, Optional, Type

import swapper

from django.conf import Settings, settings
from django.contrib.auth import get_user_model

from django_project_base.constants import (
    EMAIL_SENDER_ID_SETTING_NAME,
    SMS_SENDER_ID_SETTING_NAME,
    USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER,
)
from django_project_base.notifications.base.channels.channel import Channel
from django_project_base.notifications.base.duplicate_notification_mixin import DuplicateNotificationMixin
from django_project_base.notifications.base.enums import ChannelIdentifier, NotificationLevel, NotificationType
from django_project_base.notifications.base.queable_notification_mixin import QueableNotificationMixin
from django_project_base.notifications.base.send_notification_service import SendNotificationService
from django_project_base.notifications.models import DjangoProjectBaseMessage, DjangoProjectBaseNotification


class Notification(QueableNotificationMixin, DuplicateNotificationMixin):
    _persist = False
    _delay = None
    _recipients = []
    _extra_data = {}
    type = NotificationType.STANDARD.value
    level = None
    locale = None
    message: DjangoProjectBaseMessage
    content_entity_context = ""
    send_notification_sms = False
    _raw_recipents: str
    _project: Optional[object]

    _via_channels = List[Type[Channel]]
    _user = None

    def __init__(
        self,
        message: DjangoProjectBaseMessage,
        raw_recipents: List[str],
        project,
        persist: bool = False,
        level: Optional[NotificationLevel] = None,
        locale: Optional[str] = None,
        delay: Optional[int] = None,
        type: Optional[NotificationType] = None,
        recipients=None,
        content_entity_context="",
        channels=[],
        user=None,
        send_notification_sms=False,
        **kwargs,
    ) -> None:
        super().__init__()
        if recipients is None:
            recipients = []
        if type is None:
            type = NotificationType.STANDARD
        if level is None:
            level = NotificationLevel.INFO
        self.send_notification_sms = send_notification_sms
        assert isinstance(persist, bool), "Persist must be valid boolean value"
        assert raw_recipents is not None, "Original recipients payload is required"
        self._raw_recipents = json.dumps(raw_recipents)
        self._persist = persist
        if level is not None:
            lvl = level.value if isinstance(level, NotificationLevel) else level
            assert lvl in [_level.value for _level in NotificationLevel], "Invalid notification level value"
            self.level = level if isinstance(level, NotificationLevel) else NotificationLevel(lvl)
        self.locale = locale
        if delay is not None:
            self._delay = delay
        if type is not None:
            typ = type.value if isinstance(type, NotificationType) else type
            assert typ in [t.value for t in NotificationType], "Invalid notification type value"
            self.type = type if isinstance(type, NotificationType) else NotificationType(typ)
        assert isinstance(recipients, list), "Recipients must be a list"
        self._recipients = recipients
        self._extra_data = kwargs or {}
        assert isinstance(message, DjangoProjectBaseMessage), "Invalid value for message"
        self.message = message
        self.content_entity_context = content_entity_context
        if channels:
            # TODO: check for supported channels
            self.via_channels = channels
        self._project = project
        self._user = user

    @staticmethod
    def resend(notification: DjangoProjectBaseNotification, user_pk: Optional[str] = None):
        if user_pk == "None":
            user_pk = None
        notification.user = user_pk or (notification.extra_data or {}).get("user")
        from django_project_base.notifications.rest.notification import MessageToListField

        recipients: List[str] = MessageToListField.parse(json.loads(notification.recipients_original_payload))
        notification.recipients = ",".join(map(str, recipients)) if recipients else None
        notification.recipients_original_payload_search = None
        notification.sender = Notification._get_sender_config(notification.project_slug)
        mail_fallback: bool = (
            swapper.load_model("django_project_base", "ProjectSettings")
            .objects.get(name=USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER, project__slug=notification.project_slug)
            .python_value
            if notification.project_slug
            else False
        )
        notification.email_fallback = mail_fallback
        notification.save(update_fields=["recipients", "recipients_original_payload_search"])
        SendNotificationService(settings=settings, use_default_db_connection=True).make_send(
            notification, {}, resend=True
        )

    def __set_via_channels(self, val):
        self._via_channels = val

    def __get_via_channels(self):
        return self._via_channels

    via_channels = property(__get_via_channels, __set_via_channels)

    @property
    def delay(self) -> Optional[int]:
        return self._delay

    @property
    def persist(self) -> bool:
        return bool(self._persist)

    @staticmethod
    def _get_sender_config(project_slug: Optional[str]) -> dict:
        from django_project_base.notifications.base.channels.mail_channel import MailChannel
        from django_project_base.notifications.base.channels.sms_channel import SmsChannel

        if project_slug and (
            project := swapper.load_model("django_project_base", "Project").objects.filter(slug=project_slug).first()
        ):
            project_settings_model = swapper.load_model("django_project_base", "ProjectSettings")
            mail_settings = project_settings_model.objects.filter(
                name=EMAIL_SENDER_ID_SETTING_NAME, project=project
            ).first()
            sms_settings = project_settings_model.objects.filter(
                name=SMS_SENDER_ID_SETTING_NAME, project=project
            ).first()
            return {
                MailChannel.name: mail_settings.python_value if mail_settings else "",
                SmsChannel.name: sms_settings.python_value if sms_settings else "",
            }
        return {
            MailChannel.name: "",
            SmsChannel.name: "",
        }

    def send(self) -> DjangoProjectBaseNotification:
        required_channels: list = list(
            map(lambda f: str(f), filter(lambda d: d is not None, map(lambda c: c.name, self.via_channels)))
        )
        self._extra_data["user"] = self._user
        notification: DjangoProjectBaseNotification = DjangoProjectBaseNotification(
            locale=self.locale,
            level=self.level.value,
            delayed_to=self.delay,
            required_channels=",".join(required_channels) if required_channels else None,
            type=self.type.value,
            message=self.message,
            content_entity_context=str(self.content_entity_context)
            if self.content_entity_context
            else str(uuid.uuid4()),
            recipients=",".join(map(str, self._recipients)) if self._recipients else None,
            recipients_original_payload=self._raw_recipents,
            project_slug=self._project,
            send_notification_sms=self.send_notification_sms,
            send_notification_sms_text=None,
            extra_data=self._extra_data,
        )

        notification = self._ensure_channels(channels=required_channels, notification=notification, settings=settings)

        required_channels.sort()
        if self.persist:
            if self.handle_similar_notifications(notification=notification):
                return notification
            if not self.message.pk or not DjangoProjectBaseMessage.objects.filter(pk=self.message.pk).exists():
                self.message.save()
            notification.created_at = int(datetime.datetime.now().timestamp())
            notification.save()
        else:
            notification.created_at = None

        if self.delay:
            if not self.persist:
                raise Exception("Delayed notification must be persisted")

            rec_list = self._extra_data.get("a_recipients_list") or []
            if len(rec_list) == 0:
                for usr in self._recipients:
                    rec_list.append(
                        {
                            k: v
                            for k, v in get_user_model().objects.get(pk=usr).userprofile.__dict__.items()
                            if not k.startswith("_")
                        }
                    )
            notification.recipients_list = rec_list
            ext_data = self._extra_data.get("a_extra_data") or self._extra_data
            notification.extra_data = ext_data
            notification.save(update_fields=["extra_data"])
            self.enqueue_notification(notification, extra_data=ext_data)
            return notification

        if not self.delay:
            SendNotificationService(settings=settings, use_default_db_connection=True).make_send(
                notification, self._extra_data, resend=False
            )
        else:
            if not self.persist:
                raise Exception("Delayed notification must be persisted")
            mail_fallback: bool = (
                swapper.load_model("django_project_base", "ProjectSettings")
                .objects.get(name=USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER, project__slug=notification.project_slug)
                .python_value
                if notification.project_slug
                else False
            )
            rec_list = []
            for usr in self._recipients:
                rec_list.append(
                    {
                        k: v
                        for k, v in get_user_model().objects.get(pk=usr).userprofile.__dict__.items()
                        if not k.startswith("_") and isinstance(v, (str, list, tuple, int, float))
                    }
                )
            notification.extra_data["recipients-list"] = rec_list
            notification.extra_data["sender"] = notification.sender
            notification.extra_data["mail-fallback"] = mail_fallback
            notification.save(update_fields=["extra_data"])

        return notification

    def _ensure_channels(
        self,
        channels: List[str],
        notification: DjangoProjectBaseNotification,
        settings: Optional[Settings] = None,
    ) -> DjangoProjectBaseNotification:
        from django_project_base.notifications.base.channels.mail_channel import MailChannel

        extra_data = self._extra_data.get("a_extra_data") or self._extra_data
        settings = self._extra_data.get("a_settings") or settings
        for channel_name in channels:
            # ensure dlr user and check providers
            channel = ChannelIdentifier.channel(channel_name, settings=settings, project_slug=self._project)

            if not channel and extra_data.get("is_system_notification"):
                continue

            assert channel

            if self.send_notification_sms and channel.name == MailChannel.name:
                notification.send_notification_sms_text = channel.provider.get_send_notification_sms_text(
                    notification=notification, host_url=extra_data.get("host_url", "")  # noqa: E126
                )

        notification.user = self._user

        notification.sender = self._extra_data.get("a_sender") or Notification._get_sender_config(self._project)

        mail_fallback = False
        if not self._extra_data.get("a_sender"):
            mail_fallback: bool = (
                swapper.load_model("django_project_base", "ProjectSettings")
                .objects.get(name=USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER, project__slug=notification.project_slug)
                .python_value
                if notification.project_slug
                else False
            )
        notification.email_fallback = mail_fallback

        if self._extra_data.get("is_system_notification"):
            notification.sender[MailChannel.name] = getattr(settings, "SYSTEM_EMAIL_SENDER_ID", "")
            from django_project_base.notifications.base.channels.sms_channel import SmsChannel

            notification.sender[SmsChannel.name] = getattr(settings, "SYSTEM_SMS_SENDER_ID", "")

        return notification



from django.conf import settings
from django.core.cache import cache
from django.utils.translation import gettext as __
from natural.date import compress
from rest_framework.request import Request

from django_project_base.notifications.email_notification import SystemEMailNotification
from django_project_base.notifications.models import DjangoProjectBaseMessage


def send_register_verification_email_notification(
    request: Request,
    user,
) -> None:
    if (flow_id := request.COOKIES.get("register-flow")) and (code := cache.get(flow_id)):
        SystemEMailNotification(
            message=DjangoProjectBaseMessage(
                subject=f"{__('Account confirmation for')} {request.META['HTTP_HOST']}",
                body=f"{__('You or someone acting as you registered an account at')} "
                f"{request.META['HTTP_HOST']}. "
                f"\n\n{__('Your verification code is')}: "
                f"{code} \n\n {__('Code is valid for')} {compress(settings.CONFIRMATION_CODE_TIMEOUT)}.\n\n"
                f"{__('If this was not you or it was unintentional, you may safely ignore this message.')}",
                footer="",
                content_type=DjangoProjectBaseMessage.PLAIN_TEXT,
            ),
            recipients=[user.pk],
            user=user.pk,
        ).send()
        cache.set(code, user, timeout=settings.CONFIRMATION_CODE_TIMEOUT)



# Generated by Django 4.2.4 on 2023-09-01 10:21

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0002_remove_djangoprojectbasenotification_project_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="djangoprojectbasemessage",
            options={"verbose_name": "Notification"},
        ),
    ]



from django.apps import AppConfig

from django_project_base.account import ACCOUNT_APP_ID
from django_project_base.account.settings import ACCOUNT_SETTINGS
from django_project_base.settings_parser import parse_settings


class DjangoProjectBaseAccountsConfig(AppConfig):
    name = ACCOUNT_APP_ID
    verbose_name = "Django Project Base Account management"

    def ready(self):
        parse_settings(ACCOUNT_SETTINGS)



# Generated by Django 4.2.4 on 2023-09-20 08:16
from gettext import gettext

import swapper
from django.db import migrations, models

from django_project_base.constants import SEND_NOTIFICATION_SMS


def forwards_func(apps, schema_editor):
    project_sett = swapper.load_model("django_project_base", "ProjectSettings")
    for project in swapper.load_model("django_project_base", "Project").objects.all():
        project_sett.objects.get_or_create(
            project=project,
            name=SEND_NOTIFICATION_SMS,
            defaults=dict(
                description=gettext("Text for notification detail view. __LINK__ is url placeholder."),
                value=gettext("You received a notification. Clik link to view it: __LINK__"),
                value_type="char",
            ),
        )


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0005_merge_20230906_1213"),
    ]

    operations = [
        migrations.AddField(
            model_name="djangoprojectbasenotification",
            name="send_notification_sms",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="djangoprojectbasenotification",
            name="send_notification_sms_text",
            field=models.TextField(blank=False, null=True),
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



import swapper

from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from dynamicforms import fields
from dynamicforms.action import Actions, TableAction, TablePosition
from dynamicforms.mixins import DisplayMode
from dynamicforms.serializers import ModelSerializer
from dynamicforms.viewsets import ModelViewSet
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.request import Request

from django_project_base.base.event import UserInviteFoundEvent
from django_project_base.base.exceptions import InviteActionNotImplementedException
from django_project_base.base.viewsets import ProjectFilteringViewSet
from django_project_base.constants import INVITE_NOTIFICATION_TEXT
from django_project_base.notifications.email_notification import EMailNotificationWithListOfEmails
from django_project_base.notifications.models import DjangoProjectBaseMessage
from django_project_base.notifications.rest.notification import MessageBodyField
from django_project_base.utils import get_host_url, get_pk_name


class AcceptedField(fields.BooleanField):
    def __init__(self, **kw):
        super().__init__(**kw)
        self.read_only = True

    def to_representation(self, value, row_data=None):
        if row_data and row_data.accepted:
            return True
        return False


class InviteTextField(MessageBodyField):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        self.label = _("Invitation text")
        self.help_text = _(
            "Use %LINK% as a placeholder for invite link. If invite text is empty, value from settings will be used"
        )
        self.allow_blank = True


class ProjectUserInviteSerializer(ModelSerializer):
    template_context = dict(url_reverse="project-user-invite")
    form_titles = {
        "table": _("Project invites"),
        "new": _("Inviting project member"),
        "edit": "",
    }

    actions = Actions(
        TableAction(TablePosition.HEADER, _("Add"), title=_("Add new record"), name="add", icon="add-circle-outline"),
        add_form_buttons=True,
        add_default_filter=True,
        add_default_crud=False,
    )

    id = fields.AutoGeneratedField(display=DisplayMode.HIDDEN)
    invited_by = fields.AutoGeneratedField(display=DisplayMode.HIDDEN)
    project = fields.PrimaryKeyRelatedField(
        display=DisplayMode.SUPPRESS,
        queryset=swapper.load_model("django_project_base", "Project").objects.all(),
    )
    role = fields.PrimaryKeyRelatedField(
        queryset=swapper.load_model("django_project_base", "Role").objects.all(),
        allow_null=True,
    )
    accepted = AcceptedField(display_form=DisplayMode.HIDDEN)
    text = InviteTextField()

    class Meta:
        model = swapper.load_model("django_project_base", "Invite")
        exclude = ()
        fields = (
            get_pk_name(swapper.load_model("django_project_base", "Invite")),
            "email",
            "role",
            "accepted",
            "text",
            "project",
            "invited_by",
        )


class ProjectUserInviteViewSet(ProjectFilteringViewSet):
    # TODO: ADD INVITE SHOULD BE PERMISSION BASED ON ROLE
    permission_classes = (IsAuthenticated,)
    serializer_class = ProjectUserInviteSerializer
    model = staticmethod(lambda: swapper.load_model("django_project_base", "Invite"))

    def get_permissions(self):
        if self.action == "accept":
            return [IsAuthenticated()]
        else:
            return super().get_permissions()

    def new_object(self: ModelViewSet):
        new_object = super().new_object()
        new_object.project = self.request.selected_project
        return new_object

    @transaction.atomic
    def update(self, request, *args, **kwargs):
        self.request.data["project"] = self.request.selected_project.pk
        self.request.data["invited_by"] = self.request.user.userprofile
        created = super().create(request, *args, **kwargs)
        invite_url = (
            f"{get_host_url(request)}account/project-user-invite/"
            f"{created.data[get_pk_name(self.get_serializer_class().Meta.model)]}/accept"
        )
        invite_text = request.data.get("text")
        if not invite_text or "%LINK%" not in invite_text:
            invite_text = (
                swapper.load_model("django_project_base", "ProjectSettings")
                .objects.get(project=self.request.selected_project, name=INVITE_NOTIFICATION_TEXT)
                .python_value
            )
            invite_text = invite_text.replace("__LINK__", invite_url)
        else:
            invite_text = invite_text.replace("%LINK%", invite_url)

        EMailNotificationWithListOfEmails(
            message=DjangoProjectBaseMessage(
                subject=_("You are invited to project") + f" {self.request.selected_project.name}",
                body=invite_text,
                footer="",
                content_type=DjangoProjectBaseMessage.HTML,
            ),
            recipients=[self.request.data["email"]],
            project=self.request.selected_project.slug,
            user=self.request.user.pk,
        ).send()
        return created

    @transaction.atomic
    def create(self, request, *args, **kwargs):
        raise InviteActionNotImplementedException

    @transaction.atomic
    def partial_update(self, request, *args, **kwargs):
        raise InviteActionNotImplementedException

    @transaction.atomic
    def destroy(self, request, *args, **kwargs):
        raise InviteActionNotImplementedException

    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @extend_schema(exclude=True)
    @action(methods=["GET"], detail=True, url_name="accept", url_path="accept")
    def accept(self, request: Request, pk: str, *args, **kwargs) -> HttpResponse:
        invite = get_object_or_404(swapper.load_model("django_project_base", "Invite"), pk=pk)
        response = HttpResponseRedirect(get_host_url(request))
        if not invite.accepted:
            if request.user.is_authenticated:
                UserInviteFoundEvent(request.user).trigger(payload=invite, request=request)
                return response
            response.set_cookie("invite-pk", pk)
            request.session["invite-pk"] = pk
        return response



ACCOUNT_URL_PREFIX = "account"

NOTIFICATION_QUEUE_NAME = "notification"

EMAIL_SENDER_ID_SETTING_NAME = "email-sender-id"
SMS_SENDER_ID_SETTING_NAME = "sms-sender-id"

SEND_NOTIFICATION_SMS = "notification-sms-link-text"

INVITE_NOTIFICATION_TEXT = "invite-notification-link-text"

NOTIFY_NEW_USER_SETTING_NAME = "notify-new-user-via-email-account-created"

USE_EMAIL_IF_RECIPIENT_HAS_NO_PHONE_NUMBER = "notify-user-via-email-if-no-phone-number"
